//  SuperTux
//  Copyright (C) 2006 DJ Gentoo <djwings@gmail.com>
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "badguy/jspiky.hpp"

#include "object/player.hpp"
#include "sprite/sprite.hpp"
#include "supertux/object_factory.hpp"

#include <algorithm>

static const float WALKSPEED = 80;

JSpiky::JSpiky(const Reader& reader)
	: WalkingBadguy(reader, "images/creatures/spiky/jumpingspiky.sprite", "left", "right"), state(JSPIKY_WALKING)
{
  walk_speed = WALKSPEED;
  max_drop_height = 600;
}

HitResponse
JSpiky::collision_badguy(BadGuy& badguy, const CollisionHit& hit)
{
  if(state != JSPIKY_WALKING) {
    return BadGuy::collision_badguy(badguy, hit);
  }
  return WalkingBadguy::collision_badguy(badguy, hit);
}

void
JSpiky::active_update(float elapsed_time) {

  if(state == JSPIKY_JUMPING) {
    if(physic.get_velocity_y() >= 500) {
      physic.set_velocity_y(0);
      sprite->set_action(dir == LEFT ? "walking-left" : "walking-right", 1);
      state = JSPIKY_WALKING;
      WalkingBadguy::initialize();
    }
    return;
  }

  if(state == JSPIKY_WALKING) {

    Player* player = get_nearest_player();
    if (player) {
      Rectf pb = player->get_bbox();

      bool inReach_left = (pb.p2.x >= bbox.p2.x-((dir == LEFT) ? 256 : 0));
      bool inReach_right = (pb.p1.x <= bbox.p1.x+((dir == RIGHT) ? 256 : 0));
      bool inReach_top = (pb.p2.y >= bbox.p1.y);
      bool inReach_bottom = (pb.p1.y <= bbox.p2.y);

      if (inReach_left && inReach_right && inReach_top && inReach_bottom) {
        // wake up
        sprite->set_action(dir == LEFT ? "crouching-left" : "crouching-right", 1);
        state = JSPIKY_CROUCHING;
        walk_speed = 0;
      }
    }

    BadGuy::active_update(elapsed_time);
  }

  if(state == JSPIKY_CROUCHING) {
    if(sprite->animation_done()) {
      // start jumping
      state = JSPIKY_JUMPING;
      physic.set_velocity_y(frozen ? 0 : -500);
    }

    BadGuy::active_update(elapsed_time);
  }
}

void
JSpiky::freeze()
{
  WalkingBadguy::freeze();
  state = JSPIKY_WALKING; // if we get hit while sleeping, wake up :)
}

bool
JSpiky::is_freezable() const
{
  return true;
}

/* EOF */
